-- tourney mode commands

-- commands to change team name and colours
COM_AddCommand("tsr_tourneycolor", function(p, colour, team)
	if not (server.tsr_server.tourneymode) then CONS_Printf(p, "\x83NOTICE:\x80 This can only be used in Tourney Mode setup!") return end
	
	team = tonumber($)
	local isAdmin = (server == p or IsPlayerAdmin(p))
	if (team and (team >= 1 and team <= 5))
	and (isAdmin)
		-- check colour
		colour = K_GetKartColorByName($)
		if not (colour >= 1 and colour <= SKINCOLOR_LILAC) then CONS_Printf(p, "\x82WARNING:\x80 Invalid team color given!") return end
		
		-- make sure we dont choose a similar colour to any other team
		local colourmap = tsrb2kr.ColorToMap(colour)
		for i = 1, 5
			if (i == p.tsr.tourneyteam) then continue end
			local team_colourmap = tsrb2kr.ColorToMap(server.tsr_tourneycolours[i])
			
			if (colourmap == team_colourmap)
				CONS_Printf(p, "\x82WARNING:\x80 Color is too similar to Team "..i.."'s color!")
				return
			end
		end
	
		server.tsr_tourneycolours[team] = colour
		return
	end
	
	-- team checks
	if not (p.tsr.tourneyteam) then CONS_Printf(p, "\x83NOTICE:\x80 You need to be on a team for this!") return end
	if (p != server.tsr_tourneyteams[p.tsr.tourneyteam][1]) then CONS_Printf(p, "\x83NOTICE:\x80 Only the team leader can do this!") return end
	
	-- check colour
	if not (colour) then CONS_Printf(p, "\x82WARNING:\x80 No team color given!") return end
	colour = K_GetKartColorByName($)
	if not (colour >= 1 and colour <= SKINCOLOR_LILAC) then CONS_Printf(p, "\x82WARNING:\x80 Invalid team color given!") return end
	
	-- make sure we dont choose a similar colour to any other team
	local colourmap = tsrb2kr.ColorToMap(colour)
	for i = 1, 5
		if (i == p.tsr.tourneyteam) then continue end
		local team_colourmap = tsrb2kr.ColorToMap(server.tsr_tourneycolours[i])
		
		if (colourmap == team_colourmap)
			CONS_Printf(p, "\x82WARNING:\x80 Color is too similar to Team "..i.."'s color!")
			return
		end
	end
	
	server.tsr_tourneycolours[p.tsr.tourneyteam] = colour
end)
COM_AddCommand("tsr_tourneyname", function(p, name, team)
	if not (server.tsr_server.tourneymode) then CONS_Printf(p, "\x83NOTICE:\x80 This can only be used in Tourney Mode setup!") return end
	
	team = tonumber($)
	local isAdmin = (server == p or IsPlayerAdmin(p))
	if (team and (team >= 1 and team <= 5))
	and (isAdmin)
		server.tsr_tourneynames[team] = name
		return
	end
	
	-- team checks
	if not (p.tsr.tourneyteam) then CONS_Printf(p, "\x83NOTICE:\x80 You need to be on a team for this!") return end
	if (p != server.tsr_tourneyteams[p.tsr.tourneyteam][1]) then CONS_Printf(p, "\x83NOTICE:\x80 Only the team leader can do this!") return end
	
	-- check name, then set it
	if not (name) then CONS_Printf(p, "\x82WARNING:\x80 No team name given!") return end
	server.tsr_tourneynames[p.tsr.tourneyteam] = name
end)

-- tourney mode blacklist, get rid of stages from the rotation!
tsrb2kr.IsMapBlacklisted = function(mapnum)
	if not (mapheaderinfo[mapnum]) then return end
	
	for k, v in ipairs(server.tsr_tourneyblacklist)
		if (v != mapnum) then continue end
					
		-- if we're here, the mapnum is already added
		return true
	end
	
	return false
end
COM_AddCommand("tsr_tourneyblacklist", function(p, option, ...)
	local isAdmin = (server == p or IsPlayerAdmin(p))
	local options = (isAdmin) and "<view/add/remove/clear>" or "<view>"
	
	-- make sure they selected an option
	if not (option) then CONS_Printf(p, "tsr_tourneyblacklist "..options) return end
	
	-- non admin options
	if (option == "view")	-- print the rotation!
		CONS_Printf(p, "Current tourney blacklist:")
		for k, v in ipairs(server.tsr_tourneyblacklist)
			-- build levelname
			local info = mapheaderinfo[v]
			local levelname = info.lvlttl
			if (info.zonttl:gsub(" ", ""):len() and not (info.levelflags & LF_NOZONE)) then levelname = $ + " "..info.zonttl end
			if (info.actnum:gsub(" ", ""):len()) then levelname = $ + " "..info.actnum end
			
			CONS_Printf(p, "\x82"..k..":\x80 "..G_BuildMapName(v).." - "..levelname)
		end
		
		return
	end
	
	if (isAdmin)
		if (option == "add")
			local args = {...}
			if not (args[1]) then CONS_Printf(p, "tsr_tourneyblacklist add <map> <range>") return end
			args[1] = $:upper()
			
			-- check mapnum
			local mapnum = tsrb2kr.StringToMapnum(args[1]:sub(4))
			if (G_BuildMapName(mapnum) != args[1]) then CONS_Printf(p, "\x82WARNING:\x80 Invalid map! (Must be formatted like MAPxx)") return end
			-- ^ no, you CANNOT exploit :sub and put ASS01 to make it MAP01
			if not (mapnum) then CONS_Printf(p, "\x82WARNING:\x80 Invalid map! (Must be formatted like MAPxx)") return end
			
			-- get range
			local range = mapnum
			if (args[2])
				args[2] = $:upper()
				
				local mapnum2 = tsrb2kr.StringToMapnum(args[2]:sub(4))
				if (mapnum2 and G_BuildMapName(mapnum2) == args[2])
					if (mapnum2 < range)	-- bruh
						mapnum = mapnum2
					else
						range = mapnum2
					end
				else
					CONS_Printf(p, "\x82WARNING:\x80 Invalid map range! (Must be formatted like MAPxx)")
				end
			end
			
			for i = 0, abs(range - mapnum)
				local map = mapnum + i
				if not (mapheaderinfo[map]) then CONS_Printf(p, "\x82WARNING:\x80 "..G_BuildMapName(map).." doesn't exist!") continue end
				
				-- build levelname
				local info = mapheaderinfo[map]
				local levelname = info.lvlttl
				if (info.zonttl:gsub(" ", ""):len() and not (info.levelflags & LF_NOZONE)) then levelname = $ + " "..info.zonttl end
				if (info.actnum:gsub(" ", ""):len()) then levelname = $ + " "..info.actnum end
				
				-- battle? nope
				if (info.typeoflevel & TOL_MATCH) then CONS_Printf(p, "\x82WARNING:\x80 "..levelname.." is a battle map!") continue end
				
				-- check if the map is already added through mapnum checks
				local alreadyAdded
				for k, v in ipairs(server.tsr_tourneyblacklist)
					if (v != map) then continue end
					
					-- if we're here, the mapnum is already added
					alreadyAdded = k
					break
				end
				if (alreadyAdded) then CONS_Printf(p, "\x82WARNING:\x80 "..levelname.." is already added! (Index "..alreadyAdded..")") continue end
				
				CONS_Printf(p, "Added "..G_BuildMapName(map).." - "..levelname.." to the tourney blacklist.")
				table.insert(server.tsr_tourneyblacklist, map)
				table.sort(server.tsr_tourneyblacklist, function(a, b) return (a < b) end)	-- just be sure to sort it based on mapnum
			end
			
			return
		elseif (option == "remove")
			local args = {...}
			if not (args[1]) then CONS_Printf(p, "tsr_tourneyblacklist remove <index> <range>") return end
			
			local index = tonumber(args[1])
			if not (index) then CONS_Printf(p, "\x82WARNING:\x80 Invalid blacklist table index!") return end
			
			-- get range
			local range = index
			if (args[2])
				local index2 = tonumber(args[2])
				if (index2)
					if (index2 < range)	-- bruh
						index = index2
					else
						range = index2
					end
				else
					CONS_Printf(p, "\x82WARNING:\x80 Invalid blacklist table index range!")
				end
			end
			
			for i = 0, abs(range - index)
				local mapindex = index + i
				if not (server.tsr_tourneyblacklist[index]) then CONS_Printf(p, "\x82WARNING:\x80 Blacklist table index "..mapindex.." doesn't exist!") continue end
				
				-- build levelname
				local mapnum = server.tsr_tourneyblacklist[mapindex]
				local info = mapheaderinfo[mapnum]
				local levelname = info.lvlttl
				if (info.zonttl:gsub(" ", ""):len() and not (info.levelflags & LF_NOZONE)) then levelname = $ + " "..info.zonttl end
				if (info.actnum:gsub(" ", ""):len()) then levelname = $ + " "..info.actnum end
				
				CONS_Printf(p, "Removed tourney blacklist index "..mapindex..": "..G_BuildMapName(mapnum).." - "..levelname)
				table.remove(server.tsr_tourneyblacklist, mapindex)
				table.sort(server.tsr_tourneyblacklist, function(a, b) return (a < b) end)	-- just be sure to sort it based on mapnum
			end
			
			return
		elseif (option == "clear")
			CONS_Printf(p, "Cleared the tourney blacklist!")
			server.tsr_tourneyblacklist = {}
			return
		end
	end
	
	-- we didnt get a single option?
	CONS_Printf(p, "tsr_tourneyblacklist "..options)
end)

-- everything to do with the tourney rotation
COM_AddCommand("tsr_tourneyrotation", function(p, option, ...)
	local isAdmin = (server == p or IsPlayerAdmin(p))
	local options = (isAdmin) and "<view/add/remove/clear>" or "<view>"
	
	-- make sure they selected an option
	if not (option) then CONS_Printf(p, "tsr_tourneyrotation "..options) return end
	
	-- non admin options
	if (option == "view")	-- print the rotation!
		CONS_Printf(p, "Current tourney rotation:")
		for k, v in ipairs(server.tsr_tourneyrotation)
			-- build levelname
			local info = mapheaderinfo[v]
			local levelname = info.lvlttl
			if (info.zonttl:gsub(" ", ""):len() and not (info.levelflags & LF_NOZONE)) then levelname = $ + " "..info.zonttl end
			if (info.actnum:gsub(" ", ""):len()) then levelname = $ + " "..info.actnum end
			
			CONS_Printf(p, "\x82"..k..":\x80 "..G_BuildMapName(v).." - "..levelname)
		end
		
		return
	end
	
	if (isAdmin)
		if (option == "add")
			local args = {...}
			if not (args[1]) then CONS_Printf(p, "tsr_tourneyrotation add <map> <range>") return end
			args[1] = $:upper()
			
			-- check mapnum
			local mapnum = tsrb2kr.StringToMapnum(args[1]:sub(4))
			if (G_BuildMapName(mapnum) != args[1]) then CONS_Printf(p, "\x82WARNING:\x80 Invalid map! (Must be formatted like MAPxx)") return end
			-- ^ no, you CANNOT exploit :sub and put ASS01 to make it MAP01
			if not (mapnum) then CONS_Printf(p, "\x82WARNING:\x80 Invalid map! (Must be formatted like MAPxx)") return end
			
			-- get range
			local range = mapnum
			if (args[2])
				args[2] = $:upper()
				
				local mapnum2 = tsrb2kr.StringToMapnum(args[2]:sub(4))
				if (mapnum2 and G_BuildMapName(mapnum2) == args[2])
					if (mapnum2 < range)	-- bruh
						mapnum = mapnum2
					else
						range = mapnum2
					end
				else
					CONS_Printf(p, "\x82WARNING:\x80 Invalid map range! (Must be formatted like MAPxx)")
				end
			end
			
			for i = 0, abs(range - mapnum)
				local map = mapnum + i
				if not (mapheaderinfo[map]) then CONS_Printf(p, "\x82WARNING:\x80 "..G_BuildMapName(map).." doesn't exist!") continue end
				
				-- build levelname
				local info = mapheaderinfo[map]
				local levelname = info.lvlttl
				if (info.zonttl:gsub(" ", ""):len() and not (info.levelflags & LF_NOZONE)) then levelname = $ + " "..info.zonttl end
				if (info.actnum:gsub(" ", ""):len()) then levelname = $ + " "..info.actnum end
				
				-- battle? nope
				if (info.typeoflevel & TOL_MATCH) then CONS_Printf(p, "\x82WARNING:\x80 "..levelname.." is a battle map!") continue end
				
				-- check if the map is already added through mapnum checks
				local alreadyAdded
				for k, v in ipairs(server.tsr_tourneyrotation)
					if (v != map) then continue end
					
					-- if we're here, the mapnum is already added
					alreadyAdded = k
					break
				end
				if (alreadyAdded) then CONS_Printf(p, "\x82WARNING:\x80 "..levelname.." is already added! (Index "..alreadyAdded..")") continue end
				
				CONS_Printf(p, "Added "..G_BuildMapName(map).." - "..levelname.." to the tourney rotation.")
				table.insert(server.tsr_tourneyrotation, map)
				table.sort(server.tsr_tourneyrotation, function(a, b) return (a < b) end)	-- just be sure to sort it based on mapnum
			end
			
			return
		elseif (option == "remove")
			local args = {...}
			if not (args[1]) then CONS_Printf(p, "tsr_tourneyrotation remove <index> <range>") return end
			
			local index = tonumber(args[1])
			if not (index) then CONS_Printf(p, "\x82WARNING:\x80 Invalid rotation table index!") return end
			
			-- get range
			local range = index
			if (args[2])
				local index2 = tonumber(args[2])
				if (index2)
					if (index2 < range)	-- bruh
						index = index2
					else
						range = index2
					end
				else
					CONS_Printf(p, "\x82WARNING:\x80 Invalid rotation table index range!")
				end
			end
			
			for i = 0, abs(range - index)
				local mapindex = index + i
				if not (server.tsr_tourneyrotation[index]) then CONS_Printf(p, "\x82WARNING:\x80 Rotation table index "..mapindex.." doesn't exist!") continue end
				
				-- build levelname
				local mapnum = server.tsr_tourneyrotation[mapindex]
				local info = mapheaderinfo[mapnum]
				local levelname = info.lvlttl
				if (info.zonttl:gsub(" ", ""):len() and not (info.levelflags & LF_NOZONE)) then levelname = $ + " "..info.zonttl end
				if (info.actnum:gsub(" ", ""):len()) then levelname = $ + " "..info.actnum end
				
				CONS_Printf(p, "Removed tourney rotation index "..mapindex..": "..G_BuildMapName(mapnum).." - "..levelname)
				table.remove(server.tsr_tourneyrotation, mapindex)
				table.sort(server.tsr_tourneyrotation, function(a, b) return (a < b) end)	-- just be sure to sort it based on mapnum
			end
			
			return
		elseif (option == "clear")
			CONS_Printf(p, "Cleared the tourney rotation!")
			server.tsr_tourneyrotation = {}
			return
		end
	end
	
	-- we didnt get a single option?
	CONS_Printf(p, "tsr_tourneyrotation "..options)
end)

-- one way ticket to comp hell
COM_AddCommand("tsr_tourneymode", function(p)
	if not (server and server.valid) then return end
	
	if (server.tsr_server.tourneyround)
		CONS_Printf(p, "\x83NOTICE:\x80 A tourny round is already in progress! You can restart the map to cancel it.")
		return
	end
	if (server.tsr_server.tourneymode)
		CONS_Printf(p, "\x83NOTICE:\x80 Tourney Mode has already been activated, restart the map to deactivate it!")
		return
	end
	
	print("\x83NOTICE:\x80 Tourney Mode has been activated by the host or an admin!")
	server.tsr_server.tourneymode = true
	S_ChangeMusic("TSRTMO", true)
	
	-- init vars
	server.tsr_tourneyteams = $ or {{}, {}, {}, {}, {}}
	server.tsr_tourneynames = $ or {"Team 1", "Team 2", "Team 3", "Team 4", "Team 5"}
	server.tsr_tourneycolours = $ or {SKINCOLOR_BLUE, SKINCOLOR_RED, SKINCOLOR_GREEN, SKINCOLOR_PURPLE, SKINCOLOR_BLACK}
	-- ^ all of these are stored in server, we'll want to bring these over to the next round!
end, true)