tsrb2kr.HUD_TourneyJoiners = function(v, p, dpnum)
	for i = 1, 5
		-- draw line dividers
		if (i != 5)
			local hx = 65 + (64*(i-1))
			v.drawFill(hx, 35, 1, 36, 0)
		end
		
		local hx = 33 + (64*(i-1))
		local vflag = tsrb2kr.ColorToMap(server.tsr_tourneycolours[i], true)
		
		-- get team name
		local teamname = server.tsr_tourneynames[i]
		if (teamname:len() > 7)
			-- 15 is the limit
			local maxlen = 15
			if (teamname:len() > maxlen-1)
				teamname = $:sub(1, maxlen-1)..".."
			end
				
			v.drawString(hx-(v.stringWidth(teamname, V_6WIDTHSPACE, "thin")/2), 24, teamname, V_ALLOWLOWERCASE|V_6WIDTHSPACE|vflag, "thin")
		else
			v.drawString(hx, 24, teamname, V_ALLOWLOWERCASE|vflag, "center")
		end
		
		-- draw teammates
		local team = server.tsr_tourneyteams[i]
		for k, p in ipairs(team)
			if not (p and p.valid) then continue end
			local hy = 37 + (12*(k-1))
			
			if (p.name:len() > 7)	-- use thin font
				-- 15 is the limit
				local maxlen = 15
				local name = p.name
				if (name:len() > maxlen-1)
					name = $:sub(1, maxlen-1)..".."
				end
				
				v.drawString(hx-(v.stringWidth(name, V_6WIDTHSPACE, "thin")/2), hy, name, V_ALLOWLOWERCASE|V_6WIDTHSPACE|vflag, "thin")
			else
				v.drawString(hx, hy, p.name, V_ALLOWLOWERCASE|vflag, "center")
			end
		end
		-- draw empty slots
		if (#team < 3)
			for i = 3, (#team + 1), -1
				local hy = 37 + (12*(i-1))
				v.drawString(hx, hy, "-SLOT-", V_TRANSLUCENT|V_GRAYMAP, "center")
			end
		end
	end
end

tsrb2kr.HUD_TeamsOption = function(v, t, dpnum)
	local hilicol = V_YELLOWMAP
	if (CV_FindVar("menuhighlight").value)
		hilicol = CV_FindVar("menuhighlight").value
	elseif (modeattacking)
		hilicol = V_ORANGEMAP
	else
		hilicol = ((gametype == GT_RACE) and V_SKYMAP or V_REDMAP)
	end
	
	for i = 1, 5
		local team = t[i]
		if not (#team) then continue end
		
		local hx = 33 + (64*(i-1))
		local string = ""
		local dp_failsafe = 0
		for j = 1, #team
			local dp = team[j]
			
			-- get string for arrows
			if (dp.tsr.tourneyteam) then continue end	-- nvm, they already have a team!
			
			if (j > 1) then string = $ + " " end
			if (dpnum == 1)
				string = $ + "\x1A"
			else
				string = $ + tsrb2kr.ColorToMap(dp.skincolor).."\x1A"
			end
			dp_failsafe = $+1
		end
		
		-- draw arrows!
		if (server.tsr_server.tourneyvoting) then return end
		if not (dp_failsafe) then continue end	-- we never actually GOT anyone!
		
		local hudanim = 8 - (leveltime%7)
		v.drawString(hx, 75 - (hudanim/5), string, 0, "center")
		
		-- get team name then OVERLAP the existing one
		local teamname = server.tsr_tourneynames[i]
		if (teamname:len() > 7)
			-- 15 is the limit
			local maxlen = 15
			if (teamname:len() > maxlen-1)
				teamname = $:sub(1, maxlen-1)..".."
			end
				
			v.drawString(hx-(v.stringWidth(teamname, V_6WIDTHSPACE, "thin")/2), 24, teamname, V_ALLOWLOWERCASE|V_6WIDTHSPACE|hilicol, "thin")
		else
			v.drawString(hx, 24, teamname, V_ALLOWLOWERCASE|hilicol, "center")
		end
	end
end

tsrb2kr.HUD_VotingOption = function(v, t, dpnum)
	local hilicol = V_YELLOWMAP
	if (CV_FindVar("menuhighlight").value)
		hilicol = CV_FindVar("menuhighlight").value
	elseif (modeattacking)
		hilicol = V_ORANGEMAP
	else
		hilicol = ((gametype == GT_RACE) and V_SKYMAP or V_REDMAP)
	end
	
	-- check if any of the displayplayers are admins!
	local admin_present = false
	for dp, n in displayplayers.iterate
		if not (server == dp or IsPlayerAdmin(dp)) then continue end
		
		-- if we're here, there's an admin present
		admin_present = true
		break
	end
	
	-- no admin? only draw a greyed out option
	if not (admin_present)
	or (server.tsr_server.tourneyvoting)
		v.drawString(160, 94, "START VOTING", V_TRANSLUCENT, "center")
		return
	end
	
	local vflag = 0
	if (#t)
		vflag = hilicol
		
		-- draw cursors
		if (dpnum > 1)
			for i = 1, #t
				local dp = t[i]
				local index = tsrb2kr.IsDisplayPlayer(dp)
				local patch, colour = "P"..(index+1).."CURSOR", dp.skincolor
				
				local hx = 90 + (-10*(i-1))
				v.draw(hx, 91, v.cachePatch(patch), 0, v.getColormap(-1, colour))
			end
		else
			v.draw(90, 91, v.cachePatch("P1CURSOR"), 0, v.getColormap(-1, SKINCOLOR_WHITE))
		end
	end
	v.drawString(160, 94, "START VOTING", vflag, "center")
end

tsrb2kr.HUD_TourneyOptions = function(v)
	-- each options is a table, we'll store what displayplayers have it selected
	local displayplayer_options = {
		{{}, {}, {}, {}, {}},
		{}
	}
	local dps_found = 0
	
	for dp, n in displayplayers.iterate
		if not (dp.tsr.tourneymenu) then continue end
		
		dps_found = $+1
		if (dp.tsr.tourneymenu == 1)	-- team select, put them in one of five tables
			if not (dp.tsr.tourneyteamselect) then continue end	-- failsafe check
			
			local option = displayplayer_options[dp.tsr.tourneymenu][dp.tsr.tourneyteamselect]
			option[#option+1] = dp
		else
			local option = displayplayer_options[dp.tsr.tourneymenu]
			option[#option+1] = dp
		end
	end
	
	-- draw players on options
	for option = 1, 2
		local t = displayplayer_options[option]
		
		if (option == 1)	-- drawing teams
			tsrb2kr.HUD_TeamsOption(v, t, dps_found)
		else	-- drawing "start voting"
			tsrb2kr.HUD_VotingOption(v, t, dps_found)
		end
	end
end

tsrb2kr.HUD_TourneyVoting = function(v, p)
	if not (server.tsr_server.tourneyvoting) then return end
	
	for i = 1, 5
		local team = server.tsr_tourneyteams[i]
		if not (#team or server.tsr_server.tourneyvoted[i]) then continue end	-- they arent in the game! (or it's not buffered)
		
		local vote = server.tsr_server.tourneyvotes[i][1]
		local encore = server.tsr_server.tourneyvotes[i][2]
		
		-- get patch and levelname based on vote
		local patch, levelname
		if (vote == #server.tsr_server.tourneymaps)	-- last index is always random
			patch = "RANDOMLV"
			levelname = "RANDOM"
		else
			patch = G_BuildMapName(server.tsr_server.tourneymaps[vote]).."P"
			
			-- setup the levelname
			local info = mapheaderinfo[server.tsr_server.tourneymaps[vote]]
			levelname = info.lvlttl
			if (info.zonttl:gsub(" ", ""):len() and not (info.levelflags & LF_NOZONE)) then levelname = $ + " "..info.zonttl end
			if (info.actnum:gsub(" ", ""):len()) then levelname = $ + " "..info.actnum end
			
			-- shorten name if need be
			local maxlen = 15
			if (levelname:len() > maxlen-1)
				levelname = $:sub(1, maxlen-1)..".."
			end
		end
		patch = v.cachePatch($) or v.cachePatch("BLANKLVL")
		
		local scale = FRACUNIT/4
		local hx = 33 + (64*(i-1))
		
		local cb = {120, 5, 15, 21, 31, 22, 14, 32, 179, 202, 34, 36, 52, 54, 61, 124, 146, 151, 148, 152, 134, 127, 125, 126, 136, 137, 99, 85, 87, 127, 90, 89, 83, 88, 92, 94, 155, 85, 72, 68, 97, 113, 252, 119, 61, 106, 102, 113, 176, 107, 84, 114, 160, 168, 176, 178, 180, 182, 169, 163, 165, 167, 184, 187, 162, 186, 167, 220, 216, 214, 216, 210, 217, 215, 218, 11, 201, 202, 113, 203, 53, 205, 215, 227, 230, 235, 248, 249, 196, 192, 204, 192, 193, 126, 10, 113, 248, 254, 148, 122}
		local c = (not server.tsr_server.tourneyvoted[i] and (leveltime%10 < 4)) and 120 or cb[server.tsr_tourneycolours[i]]
		
		local flipshift, flipflag = 0, 0
		if (encore)
			flipshift = patch.width*scale
			flipflag = V_FLIP
		end
		
		v.drawFill((hx-1) - (((patch.width*scale)/2)>>FRACBITS), 104, ((patch.width*scale)>>FRACBITS)+2, ((patch.height*scale)>>FRACBITS)+2, c)
		v.drawScaled(hx<<FRACBITS - ((patch.width*scale)/2) + flipshift, 105<<FRACBITS, scale, patch, flipflag)
		v.drawString(hx-(v.stringWidth(levelname, V_6WIDTHSPACE, "thin")/2), 132, levelname, V_6WIDTHSPACE, "thin")
		
		-- encore ruby
		if (encore)
			local rubyfloattime = ((360/TICRATE)*ANG1)*(leveltime%TICRATE)
			local rubyheight = sin(rubyfloattime)
			v.drawScaled(hx<<FRACBITS, 117<<FRACBITS + (FRACUNIT/2) - rubyheight, FRACUNIT/2, v.cachePatch("RUBYICON"))
		end
		
		-- draw arrows, 1st player only
		if (server.tsr_server.tourneyvoted[i]) then continue end
		
		local dpnum, dp_control = 0, nil
		for dp, n in displayplayers.iterate
			if (dp == team[1]) then dp_control = n end
			-- ^ store what displayplayer is controlling the map selection!
			dpnum = $+1
		end
		if (dp_control == nil) then continue end
		
		local vflag = 0
		if (dpnum > 1)
			vflag = tsrb2kr.ColorToMap(team[1].skincolor, true)
		end
		
		local hudanim = 8 - (leveltime%7)
		v.drawString((hx-9) - (((patch.width*scale)/2)>>FRACBITS) - (hudanim/5), 114, "\x1C", vflag)	-- left arrow
		v.drawString((hx+1) + (((patch.width*scale)/2)>>FRACBITS) + (hudanim/5), 114, "\x1D", vflag)	-- right arrow
	end
	
	-- voting time
	if (server.tsr_server.tourneyselection) then return end	-- we arent in voting anymore
	if not (server.tsr_server.tourneyvotetime) then return end	-- failsafe
	
	local hilicol
	if (CV_FindVar("menuhighlight").value)
		hilicol = CV_FindVar("menuhighlight").value
	elseif (modeattacking)
		hilicol = V_ORANGEMAP
	else
		hilicol = ((gametype == GT_RACE) and V_SKYMAP or V_REDMAP)
	end
	local timer = server.tsr_server.tourneyvotetime/TICRATE
	v.drawString(160, 188, "VOTE ENDS IN "..timer, hilicol, "center")
end

tsrb2kr.HUD_TourneySelection = function(v)
	if not (server.tsr_server.tourneyselection) then return end
	
	local scale = FRACUNIT/2
	local patch, map, levelname
	if (server.tsr_server.tourneyvoteindex)
		map = server.tsr_server.tourneymaps[server.tsr_server.tourneyvoteindex]
		patch = G_BuildMapName(map).."P"
	else
		if not (server.tsr_server.tourneyselected) then return end	-- nope
		
		local index = server.tsr_server.tourneyvotes[server.tsr_server.tourneyselected][1]
		map = server.tsr_server.tourneymaps[index]
		
		if (map == "RANDOM")
			patch = "RANDOMLV"
			levelname = "RANDOM"
		else
			patch = G_BuildMapName(map).."P"
		end
	end
	patch = v.cachePatch($) or v.cachePatch("BLANKLVL")
	
	-- setup the levelname
	if not (levelname)
		local info = mapheaderinfo[map]
		if (info)
			levelname = info.lvlttl
			if (info.zonttl:gsub(" ", ""):len() and not (info.levelflags & LF_NOZONE)) then levelname = $ + " "..info.zonttl end
			if (info.actnum:gsub(" ", ""):len()) then levelname = $ + " "..info.actnum end
		end
	end
	
	local cb = {120, 5, 15, 21, 31, 22, 14, 32, 179, 202, 34, 36, 52, 54, 61, 124, 146, 151, 148, 152, 134, 127, 125, 126, 136, 137, 99, 85, 87, 127, 90, 89, 83, 88, 92, 94, 155, 85, 72, 68, 97, 113, 252, 119, 61, 106, 102, 113, 176, 107, 84, 114, 160, 168, 176, 178, 180, 182, 169, 163, 165, 167, 184, 187, 162, 186, 167, 220, 216, 214, 216, 210, 217, 215, 218, 11, 201, 202, 113, 203, 53, 205, 215, 227, 230, 235, 248, 249, 196, 192, 204, 192, 193, 126, 10, 113, 248, 254, 148, 122}
	local c = (not server.tsr_server.tourneyvoteindex or (leveltime%10 < 4)) and 120 or cb[server.tsr_tourneycolours[server.tsr_server.tourneymapteam]]
	v.drawFill(159-(((patch.width*scale)/2)>>FRACBITS), 143, ((patch.width*scale)>>FRACBITS)+2, ((patch.height*scale)>>FRACBITS)+2, c)
	
	local encore = server.tsr_server.tourneyencore
	local flipshift, flipflag = 0, 0
	if (encore)
		flipshift = patch.width*scale
		flipflag = V_FLIP
	end
	
	v.drawScaled(160<<FRACBITS - ((patch.width*scale)/2) + flipshift, 144<<FRACBITS, scale, patch, flipflag)
	-- encore ruby
	if (encore)
		local rubyfloattime = ((360/TICRATE)*ANG1)*(leveltime%TICRATE)
		local rubyheight = sin(rubyfloattime)
		v.drawScaled(160<<FRACBITS, 169<<FRACBITS - (rubyheight<<1), FRACUNIT, v.cachePatch("RUBYICON"))
	end
	if (levelname) then v.drawString(199-(v.stringWidth(levelname, V_6WIDTHSPACE, "thin")), 184, levelname, V_6WIDTHSPACE, "thin") end
end

tsrb2kr.HUD_GetControls = function()
	local textlines = {"- CONTROLS -"}
	
	-- check if ANY of the displayplayers meet these conditions
	local displayVoting
	local displayAdmin
	for dp, n in displayplayers.iterate
		if (server.tsr_server.tourneyvoting)
			if not (dp.tsr.tourneyteam) then continue end
			if (dp != server.tsr_tourneyteams[dp.tsr.tourneyteam][1]) then continue end
			if (server.tsr_server.tourneyvoted[dp.tsr.tourneyteam]) then continue end
			
			displayVoting = true
			break	-- no need to check anything else
		else
			if not ((server == dp) or IsPlayerAdmin(dp)) then continue end
			
			displayAdmin = true
			break
		end
	end
	
	-- vars related to checks
	if (server.tsr_server.tourneyvoting)
		if not (displayVoting) then return textlines end
		
		table.insert(textlines, "Left/Right - Cycle Maps")
		table.insert(textlines, "Up/Down - Toggle Encore")
		table.insert(textlines, "Accelerate - Select Map")
	else
		table.insert(textlines, "Left/Right - Cycle Teams")
		table.insert(textlines, "Brake - Leave Team")
		if (displayAdmin) then table.insert(textlines, "Up/Down - Cycle Options") end
		table.insert(textlines, "Accelerate - Select")
	end
	
	return textlines
end
tsrb2kr.HUD_TourneyControls = function(v)
	if not (tsrb2kr.cv_tyc.value) then return end
	if (server.tsr_server.tourneyselection) then return end
	
	local textlines = tsrb2kr.HUD_GetControls()
	if (#textlines < 2) then return end
	
	local hy = 200 - (10*(#textlines)) - ((server.tsr_server.tourneyvoting) and 10 or 0)
	for k, text in ipairs(textlines)
		local map = (k == 1) and V_YELLOWMAP or 0
		v.drawString(2, hy+(10*(k-1)), text, map|V_TRANSLUCENT)
	end
end

local BASEVIDWIDTH = 320
local SPECTEXTWIDTH = 8
tsrb2kr.HUD_TourneyUnassigned = function(v)
	local length = 0
	local height = 82
	local totallength = 0
	local templength = -8
	local dupadjust = v.width() / v.dupx()
	local duptweak = (dupadjust - BASEVIDWIDTH)/2

	for p in players.iterate
		if (p.tsr.tourneyteam) then continue end
		totallength = $+p.name:len()*SPECTEXTWIDTH + (SPECTEXTWIDTH*2)
	end

	length = $ - (leveltime % (totallength + dupadjust+SPECTEXTWIDTH))
	length = $ + dupadjust

	for p in players.iterate do
		if (p.tsr.tourneyteam) then continue end

		local le = p.name:len()*SPECTEXTWIDTH + (SPECTEXTWIDTH*2)
		if (length >= -le)
			v.drawString(length - duptweak, height, p.name, V_TRANSLUCENT|V_ALLOWLOWERCASE)
		end

		length = $+le
		if ((length + le) >= dupadjust+SPECTEXTWIDTH) then break end
    end
end

tsrb2kr.HUD_TourneyMode = function(v, p, c)
	if not (server.tsr_server.tourneymode) then return end
	hud.disable("textspectator")
	
	-- draw controls
	if (tsrb2kr.IsDisplayPlayer(p) == 0) then v.fadeScreen(65280, 16) end	-- draw the fade here, so it's behind the controls
	tsrb2kr.HUD_TourneyControls(v, p)
	if (tsrb2kr.IsDisplayPlayer(p) != 0) then return end	-- reduce rendering as much as possible
	
	-- SELECT A TEAM + lines
	v.drawString(160, 12, "* SELECT A TEAM *", 0, "center")
	local dupadjust = (v.width()/v.dupx())
	local duptweak = (dupadjust - 320)/2	-- BASEDVIDWIDTH = 320
	v.drawFill(1-duptweak, 34, dupadjust-2, 1, 0)	-- horizontal
	v.drawFill(1-duptweak, 71, dupadjust-2, 1, 0)	-- horizontal
	
	-- functions
	tsrb2kr.HUD_TourneyJoiners(v, p)
	tsrb2kr.HUD_TourneyOptions(v)
	tsrb2kr.HUD_TourneyUnassigned(v)
	tsrb2kr.HUD_TourneyVoting(v, p)
	tsrb2kr.HUD_TourneySelection(v)
end