-- tourney mode thinkers
tsrb2kr.TourneyMode_StatusSync = function()
	-- forcespec players and remove internal teams
	for p in players.iterate
		if not (p.spectator) then p.spectator = true end
		if (p.tsr.team) then p.tsr.team = nil end
	end
	
	-- make sure they have a team
	for i = 1, 5
		local team = server.tsr_tourneyteams[i]
		if not (#team) then continue end
		
		for k, v in ipairs(team)
			if not (v and v.valid)
			or not (v.tsr.tourneyteam)
				table.remove(team, k)
				if (v and v.valid)
					v.tsr.tourneyteam = nil
				end
			end
		end
	end
end

tsrb2kr.TourneyMode_CancelTeam = function(p)
	if not (p.tsr.tourneyteam) then return end
	
	if (p.tsr.inputs[BT_BRAKE] == 1)
		p.tsr.tourneyteam = nil
		S_StartSound(nil, sfx_menu1, p)
	end
end
tsrb2kr.TourneyMode_TeamSelect = function(p)
	if not (p.tsr.tourneyteamselect) then p.tsr.tourneyteamselect = 1 end
	if (p.tsr.tourneymenu != 1) then return end	-- we need to be on this
	if (p.tsr.tourneyteam) then return end	-- we already have a team
	
	-- cycle teams
	if (p.tsr.inputs["left"] == 1 or p.tsr.inputs["left"] >= 18)
	or (p.tsr.inputs["right"] == 1 or p.tsr.inputs["right"] >= 18)
		local right = (p.tsr.inputs["right"] == 1 or p.tsr.inputs["right"] >= 18)
		p.tsr.tourneyteamselect = (right) and $+1 or $-1
		
		if (p.tsr.tourneyteamselect < 1)
			p.tsr.tourneyteamselect = 5
		elseif (p.tsr.tourneyteamselect > 5)
			p.tsr.tourneyteamselect = 1
		end
		
		S_StartSound(nil, sfx_menu1, p)
	end
	
	-- select a team!
	if (p.tsr.inputs[BT_ACCELERATE] == 1)
		local team = server.tsr_tourneyteams[p.tsr.tourneyteamselect]
		
		if (#team < 3)
			-- join the team!
			p.tsr.tourneyteam = p.tsr.tourneyteamselect
			table.insert(team, p)
			
			-- play a sound
			for p2 in players.iterate
				local sound = (p == p2) and sfx_kc48 or sfx_menu1
				S_StartSound(nil, sound, p2)
			end
			
			if (p == team[1])	-- only if you're the leader
			and (p == consoleplayer)	-- only if you're the consoleplayer, too
				-- load a config, preload team settings?
				local config = io.open("tsrb2kr_tourneyleader.cfg", "r")
				if (config)
					for line in config:lines()
						COM_BufInsertText(consoleplayer, line)
					end
					CONS_Printf(consoleplayer, "\x83NOTICE:\x80 Successfully loaded tsrb2kr_tourneyleader.cfg!")
					S_StartSound(nil, sfx_tsload, consoleplayer)
					
					config:close()
				end
			end
		else
			-- team is full, pick another!
			S_StartSound(nil, sfx_lose, p)
		end
	end
end
tsrb2kr.TourneyMode_StartVoting = function(p)
	-- only admins can access this!
	if not (server == p or IsPlayerAdmin(p))
		p.tsr.tourneymenu = 1
		return
	end
	
	if (p.tsr.inputs["up"] == 1 or p.tsr.inputs["up"] >= 18)
	or (p.tsr.inputs["down"] == 1 or p.tsr.inputs["down"] >= 18)
		p.tsr.tourneymenu = (p.tsr.tourneymenu == 1) and 2 or 1
		S_StartSound(nil, sfx_menu1, p)
	end
	
	-- make sure we're on the option
	if (p.tsr.tourneymenu != 2) then return end
	
	if (p.tsr.inputs[BT_ACCELERATE] == 1)
		-- build map select table
		server.tsr_server.tourneymaps = {}
		if (#server.tsr_tourneyrotation)	-- server has it's own rotation?
			for k, v in ipairs(server.tsr_tourneyrotation)
				table.insert(server.tsr_server.tourneymaps, v)
			end
		else
			for i = 1, #mapheaderinfo
				if not (mapheaderinfo[i]) then continue end	-- not even a map?
				if (mapheaderinfo[i].typeoflevel & TOL_MATCH) then continue end	-- make sure it's not battle
				if (tsrb2kr.IsMapBlacklisted(i)) then continue end -- make sure it's not blacklisted
				
				table.insert(server.tsr_server.tourneymaps, i)
			end
		end
		table.insert(server.tsr_server.tourneymaps, "RANDOM")	-- end of the table will ALWAYS be random, keep that in mind
		
		server.tsr_server.tourneyvoting = true
		server.tsr_server.tourneyvotes = {{1, false}, {1, false}, {1, false}, {1, false}, {1, false}}	-- {vote index, encore?}
		server.tsr_server.tourneyvoted = {false, false, false, false, false}
		S_ChangeMusic("VOTE", true)
		S_StartSound(nil, sfx_menu1, p)
	end
end

tsrb2kr.TourneyMode_PlayerThinker = function(p)
	-- menu handling
	if (server.tsr_server.tourneyvoting) then return end
	if not (p.tsr.tourneymenu) then p.tsr.tourneymenu = 1 end
	
	tsrb2kr.TourneyMode_CancelTeam(p)
	tsrb2kr.TourneyMode_TeamSelect(p)
	tsrb2kr.TourneyMode_StartVoting(p)
end

tsrb2kr.TourneyMode_Voting = function()
	local total_voting, total_voted = 0, 0
	
	for i = 1, 5
		local team = server.tsr_tourneyteams[i]
		if not (#team) then continue end	-- they arent in the game!
		total_voting = $+1
		
		if (server.tsr_server.tourneyvoted[i]) then total_voted = $+1 continue end
		
		for k, v in ipairs(team)
			if (v != team[1]) then continue end	-- only the leader can do this!
			
			-- switch map
			if (v.tsr.inputs["left"] == 1 or v.tsr.inputs["left"] >= 18)
			or (v.tsr.inputs["right"] == 1 or v.tsr.inputs["right"] >= 18)
				local right = (v.tsr.inputs["right"] == 1 or v.tsr.inputs["right"] >= 18)
				server.tsr_server.tourneyvotes[i][1] = (right) and $+1 or $-1
				S_StartSound(nil, sfx_menu1, p)
				
				if (server.tsr_server.tourneyvotes[i][1] < 1)
					server.tsr_server.tourneyvotes[i][1] = #server.tsr_server.tourneymaps
				elseif (server.tsr_server.tourneyvotes[i][1] > #server.tsr_server.tourneymaps)
					server.tsr_server.tourneyvotes[i][1] = 1
				end
			end
			
			-- encore selection
			if (v.tsr.inputs["up"] == 1 or v.tsr.inputs["up"] >= 18)
			or (v.tsr.inputs["down"] == 1 or v.tsr.inputs["down"] >= 18)
				server.tsr_server.tourneyvotes[i][2] = not server.tsr_server.tourneyvotes[i][2]
				S_StartSound(nil, sfx_menu1, p)
			end
			
			if (v.tsr.inputs[BT_ACCELERATE] == 1)	-- confirm what map we'll nominate!
				server.tsr_server.tourneyvoted[i] = true
				total_voted = $+1
				S_StartSound(nil, sfx_menu1, p)
			end
		end
	end
	
	if (total_voting == total_voted)	-- we're ready, start the roulette!
		server.tsr_server.tourneyselection = true
		S_ChangeMusic("VOTEEB", false)
		return
	end
	
	-- voting timer
	local votetime = (CV_FindVar("votetime").value)*TICRATE
	local default = votetime + (votetime/2) + 1	-- 1.5 longer voting time
	
	server.tsr_server.tourneyvotetime = $ or (default)	-- init
	server.tsr_server.tourneyvotetime = $ and $-1
	if not (server.tsr_server.tourneyvotetime)	-- time's up, force random on teams that arent voting
		for i = 1, 5
			local team = server.tsr_tourneyteams[i]
			if not (#team) then continue end	-- they arent in the game!
			if (server.tsr_server.tourneyvoted[i]) then continue end	-- they did vote, good on them!
			
			-- set their vote to random
			server.tsr_server.tourneyvotes[i][1] = #server.tsr_server.tourneymaps
			server.tsr_server.tourneyvotes[i][2] = false	-- also set encore to false, you need to be functioning for this!
			server.tsr_server.tourneyvoted[i] = true
		end
		
		server.tsr_server.tourneyselection = true
		S_ChangeMusic("VOTEEB", false)
		return
	end
end

tsrb2kr.TourneyMode_Selection = function()
	server.tsr_server.tourneyselectiontimer = $ and $+1 or 1
	local timer = server.tsr_server.tourneyselectiontimer
	
	-- check for unanimous votes!
	local unanimous, unanimous_check = true, nil
	for i = 1, 5
		local team = server.tsr_tourneyteams[i]
		if not (#team) then continue end
		
		-- first check! continue to next team
		if not (unanimous_check)
			unanimous_check = server.tsr_server.tourneyvotes[i][1]
			continue
		end
		
		-- if this goes through, it's not unanimous
		if (unanimous_check != server.tsr_server.tourneyvotes[i][1])
			unanimous = false
			break
		end
	end
	
	-- instantly end selection!
	if (unanimous)
		server.tsr_server.tourneystart = true
		server.tsr_server.tourneystarting = 4*TICRATE+1
		S_StartSound(nil, sfx_kc48)
		
		-- get map through votes
		local teams = {}
		for i = 1, 5
			if not (server.tsr_server.tourneyvoted[i]) then continue end
			table.insert(teams, i)
		end
		server.tsr_server.tourneymapteam = teams[P_RandomRange(1, #teams)]
		server.tsr_server.tourneyvoteindex = server.tsr_server.tourneyvotes[server.tsr_server.tourneymapteam][1]
		server.tsr_server.tourneyencore = server.tsr_server.tourneyvotes[server.tsr_server.tourneymapteam][2]
		
		-- random handling
		if (server.tsr_server.tourneyvoteindex == #server.tsr_server.tourneymaps)
			server.tsr_server.tourneyvoteindex = P_RandomRange(1, #server.tsr_server.tourneymaps-1)
		end
		return
	end
	
	server.tsr_server.tourneycycles = $ or 0
	server.tsr_server.tourneytics = $ and $-1 or 0
	if not (server.tsr_server.tourneytics)
		if (timer >= (TICRATE + (TICRATE*45/100)))
			server.tsr_server.tourneystart = true
			server.tsr_server.tourneystarting = 3*TICRATE+1
			S_StartSound(nil, sfx_kc48)
			
			-- get map from votes
			local teams = {}
			for i = 1, 5
				if not (server.tsr_server.tourneyvoted[i]) then continue end
				table.insert(teams, i)
			end
			server.tsr_server.tourneymapteam = teams[P_RandomRange(1, #teams)]
			server.tsr_server.tourneyvoteindex = server.tsr_server.tourneyvotes[server.tsr_server.tourneymapteam][1]
			server.tsr_server.tourneyencore = server.tsr_server.tourneyvotes[server.tsr_server.tourneymapteam][2]
			
			-- random handling
			if (server.tsr_server.tourneyvoteindex == #server.tsr_server.tourneymaps)
				server.tsr_server.tourneyvoteindex = P_RandomRange(1, #server.tsr_server.tourneymaps-1)
			end
			return
		end
		
		server.tsr_server.tourneycycles = $+1
		server.tsr_server.tourneytics = min(20, (3*server.tsr_server.tourneycycles/4)+5)
		
		-- for the hud
		local possible_selections = {}
		for i = 1, 5
			if not (server.tsr_server.tourneyvoted[i]) then continue end
			if (server.tsr_server.tourneyselected == i) then continue end
			table.insert(possible_selections, i)
		end
		server.tsr_server.tourneyselected = possible_selections[P_RandomRange(1, #possible_selections)]
		S_StartSound(nil, sfx_kc39)
	end
end

tsrb2kr.TourneyMode_Start = function()
	server.tsr_server.tourneystarting = $ and $-1
	local timer = server.tsr_server.tourneystarting
	
	if (timer == 1)
		local map = server.tsr_server.tourneymaps[server.tsr_server.tourneyvoteindex]
		local encore = server.tsr_server.tourneyencore
		
		server.tsr_tourneybuffer = true
		COM_BufInsertText(server, "map "..G_BuildMapName(map)..((encore) and " -encore" or ""))	-- load the map, let's go!
		return
	end
end

tsrb2kr.Framework_TourneyMode = function(p)
	-- everything here is for rounds in progress
	if (server.tsr_server.tourneyround)
		if not (leveltime >= 1) then return end
		tsrb2kr.TourneyMode_HandleTeams()
		return
	end
	
	-- everything onwards is setting UP tourneymode.
	if not (server.tsr_server.tourneymode) then return end
	
	tsrb2kr.TourneyMode_StatusSync()
	if not (server.tsr_server.tourneyvoting)
		for p in players.iterate
			tsrb2kr.TourneyMode_PlayerThinker(p)
		end
	else
		if not (server.tsr_server.tourneyselection)
			tsrb2kr.TourneyMode_Voting()
		else
			if not (server.tsr_server.tourneystart)
				tsrb2kr.TourneyMode_Selection()
			else
				tsrb2kr.TourneyMode_Start()
			end
		end
	end
end

tsrb2kr.TourneyMode_MapLoad = function()
	if not (server.tsr_tourneybuffer)	-- we never finished the process!
		server.tsr_tourneyteams = nil
		server.tsr_tourneynames = nil
		server.tsr_tourneycolours = nil
		return
	end
	
	server.tsr_server = {}	-- setup namespace here, instead
	server.tsr_server.tourneyround = true
	server.tsr_tourneybuffer = nil
end

addHook("MusicChange", function(oldmusic, newmusic, flags, looping)
	if not (server and server.valid) then return end
	if not (server.tsr_server) then return end
	if not (server.tsr_server.tourneymode) then return end
	
	if (newmusic == "VOTE") then return end
	if (newmusic == "VOTEEB") then return end
	
	-- make sure these songs ARENT overwritten
	if (oldmusic == "TSRTMO")
	or (oldmusic == "VOTE")
	or (oldmusic == "VOTEEB")
		return oldmusic, flags, true
	end
end)